import React, { memo, useCallback, useState, useMemo, useEffect } from 'react'
import { View, Image, Text, Input, showToast } from '@ray-js/ray';
import { Popup, Slider } from "@ray-js/smart-ui";
import { useActions, useProps, useSupport, utils, useStructuredProps, useStructuredActions } from '@ray-js/panel-sdk';
import { useSelector, useDispatch } from 'react-redux';
import clsx from 'clsx';
import _ from 'lodash'

import DimmerStrip, { hsv2rgbString } from '../strip-light-smear';
import styles from './index.module.less'
import String from '@/i18n'
import { updateSmearData, selectSmearData, SmearDataType } from '@/redux/modules/uiStateSlice'
import { DimmerMode } from '@/devices/protocols/parsers/PaintColourDataFormatter';
import { SmearMode } from '@/components/Dimmer/type'
import { selectCollectSmearMap, updateCollectSmearMap, selectSmearLightColorMaps, updateSmearLightColorMaps } from '@/redux/modules/cloudStateSlice'
import { CLOUD_DATA_KEYS_MAP } from '@/constant';
import { devices } from '@/devices';
import storage from '@/api/storage'

const smearModeList = [String.getLang("smearMode_1"), String.getLang("smearMode_2"), String.getLang("smearMode_3")]

const { hsv2rgb, brightKelvin2rgb } = utils;

const COLLCET_SMEAR_KEYS = [221, 222, 223, 224, 225, 226, 227, 228, 229, 230]

const SmearLights = () => {
  const actions = useActions();
  const structuredActions = useStructuredActions();
  const dispatch = useDispatch();

  const lightNum = useProps(props => props.lightpixel_number_set);
  const paint_colour_data = useStructuredProps(props => props.paint_colour_data);

  const smearData = useSelector(selectSmearData)
  const smearLightColorMaps = useSelector(selectSmearLightColorMaps);
  const collectSmears = useSelector(selectCollectSmearMap);


  const [isShowSave, setIsShowSave] = useState(false);
  const [name, setName] = useState(''); // 保存的名称


  useEffect(() => {
    const data = {
      version: 1, // 版本号通常从 1 开始
      dimmerMode: DimmerMode.colour, // 默认为彩光模式
      effect: 0, // 默认无渐变效果
      ledNumber: 1, // 灯带段数默认为 20 段
      smearMode: SmearMode.all, // 默认涂抹模式为“油漆桶”
      hue: 0, // 彩光色相默认为 0（红色）
      saturation: 1000, // 饱和度最大值
      value: 1000, // 亮度最大值
      brightness: 1000, // 白光亮度默认中等
      temperature: 1000, // 色温默认中间值
      isColour: true, // 默认是彩光
      singleType: 0, // 默认为连续操作
      quantity: 0, // 初始没有灯带被选中
      combineType: 0, // 组合类型默认为 0
      combination: [], // 初始为空数组
      indexs: [], // 初始为空集合
    };
    Object.keys(paint_colour_data).forEach(key => {
      if (key === 'indexs') {
        data[key] = Array.from(data.indexs, (item) => item) || []
      }
      data[key] = paint_colour_data[key]
    })
    dispatch(updateSmearData(data))
  }, [])

  useEffect(() => {
    if (!isShowSave) {
      setName('');
    }
  }, [isShowSave])



  /**
  * 监听选择器的变化
  */
  const handleMearModeChange = (index: number) => {
    dispatch(updateSmearData({ ...smearData, smearMode: index }))
    structuredActions.paint_colour_data.set({
      ...smearData,
      indexs: new Set(),
      smearMode: index,
      singleType: smearData.smearMode === 0 ? 0 : 1
    })
    if (index === 0) {
      const _lightColorMaps = {}
      Object.keys(smearLightColorMaps).forEach((key) => {
        if (smearData.dimmerMode === DimmerMode.colour) {
          _lightColorMaps[key] = hsv2rgbString(
            smearData.hue,
            smearData.saturation / 10,
            smearData.value / 10,
            1
          );
        } else {
          _lightColorMaps[key] = brightKelvin2rgb(smearData.brightness, smearData.temperature);
        }
      });
      dispatch(updateSmearLightColorMaps(_lightColorMaps))
      updateCloudSmearLightColorMapsAsync(_lightColorMaps)
    }
  }

  /**
 * 监听灯带涂抹事件转变
 * @param data 
 */
  const onLightChange = (data: Set<number>) => {
    const newCheckedMapColor = {};
    [...data].forEach(item => {
      if (smearData.smearMode === SmearMode.clear) {
        newCheckedMapColor[item] = "rgba(0 ,0 ,0, 1)";
      } else {
        if (smearData.dimmerMode === DimmerMode.colour) {
          newCheckedMapColor[item] = hsv2rgbString(
            smearData.hue,
            smearData.saturation / 10,
            smearData.value / 10,
            1
          );
        } else {
          newCheckedMapColor[item] = brightKelvin2rgb(smearData.brightness, smearData.temperature);
        }
      }
    });
    const _lightColorMaps = {
      ...smearLightColorMaps,
      ...newCheckedMapColor,
    };
    structuredActions.paint_colour_data.set({
      ...smearData,
      indexs: data,
      quantity: data.size,
      singleType: smearData.smearMode === 0 ? 0 : 1
    })
    dispatch(updateSmearLightColorMaps(_lightColorMaps))
  };


  /**
* 监听灯带涂抹事件结束
* @param data 
*/
  const onLightEnd = (data: Set<number>) => {
    const newCheckedMapColor = {};
    [...data].forEach(item => {
      if (smearData.smearMode === SmearMode.clear) {
        newCheckedMapColor[item] = "rgba(0 ,0 ,0, 1)";
      } else {
        if (smearData.dimmerMode === DimmerMode.colour) {
          newCheckedMapColor[item] = hsv2rgbString(
            smearData.hue,
            smearData.saturation / 10,
            smearData.value / 10,
            1
          );
        } else {
          newCheckedMapColor[item] = brightKelvin2rgb(smearData.brightness, smearData.temperature);
        }
      }
    });
    const _lightColorMaps = {
      ...smearLightColorMaps,
      ...newCheckedMapColor,
    };
    dispatch(updateSmearLightColorMaps(_lightColorMaps))
    updateCloudSmearLightColorMapsAsync(_lightColorMaps)
  };

  /**
   * 处理点击保存涂抹
   */
  const handleSaveSmear = () => {
    if (smearData.smearMode === SmearMode.all) {
      showToast({
        title: String.getLang('save_warn'),
        icon: 'none',
        duration: 2000
      })
      return;
    }
    if (Object.keys(collectSmears).length >= 10) {
      showToast({
        title: String.getLang('save_max'),
        icon: 'none',
        duration: 2000
      })
      return;
    }
    setIsShowSave(true);
  };


  /**
   * 处理点击确认保存涂抹
   */
  const handleComfirmSave = async () => {
    if (name.length === 0) {
      showToast({
        title: String.getLang('input_error'),
        icon: 'none',
        duration: 2000
      })
    }
    const favKeys = Object.keys(collectSmears).map((item: string) => Number(item))
    const key = COLLCET_SMEAR_KEYS.find((key: number) => {
      if (!favKeys.includes(key)) return key
    })
    if (key === - 1) {
      showToast({
        title: String.getLang('save_max'),
        icon: 'none',
        duration: 2000
      })
    }
    await actions.dot_save.set(key)
    const data = { ...collectSmears, [key]: name }
    devices.lamp.model.abilities.storage.set(CLOUD_DATA_KEYS_MAP.collectSmearMap, data).then(() => {
      dispatch(updateCollectSmearMap(data));
      setIsShowSave(false);
      showToast({
        title: String.getLang('save_success'),
        icon: 'success',
      })
    }).catch((error) => {
      showToast({
        title: String.getLang('save_fail'),
        icon: 'fail',
      })
      throw new Error(error)
    })
  }

  /**
   *  颜色数据保存到云端
   */
  const updateCloudSmearLightColorMapsAsync = (data: Record<string, string>) => {
    const keys = Object.keys(data);
    const chunkSize = 20;

    // 拆分对象为多个 chunk
    const chunks: Array<Record<string, string>> = [];
    for (let i = 0; i < keys.length; i += chunkSize) {
      const chunkKeys = keys.slice(i, i + chunkSize);
      const chunkData = chunkKeys.reduce((acc, key) => {
        acc[key] = data[key];
        return acc;
      }, {} as Record<string, string>);
      chunks.push(chunkData);
    }

    return Promise.all(
      chunks.map((chunk, index) => {
        return devices.lamp.model.abilities.storage.set(CLOUD_DATA_KEYS_MAP.smearLightColorMaps + "_" + index, chunk)
      })
    ).then(() => {
      console.log('=== updateCloudSmearLightColorMapsAsync success');
    }).catch(err => {
      console.error('=== storage.set smearLightColorMaps failed', err);
    });
  }

  return (
    <View className={styles.smear_lights}>
      <View className={styles.strip_box}>
        <View className={styles.strip_top}>
          {smearModeList.map((item, index) => (
            <View className={clsx(styles.strip_mode_item, (smearData.smearMode === index ? styles.strip_mode_item_active : ''))}
              key={index}
              onClick={() => {
                handleMearModeChange(index)
              }}>
              {item}
            </View>
          ))}
          <View className={styles.save} onClick={() => {
            handleSaveSmear()
          }}>
            <Image src={require('@/static/images/common/ic_save.png')} />
            {String.getLang("save")}
          </View>
        </View>
        <View className={styles.dimmer_strip}>
          <DimmerStrip
            width={340}
            height={200}
            style={{ margin: '0 auto' }}
            disabled={false}
            lightNumber={lightNum}
            gradient={Boolean(smearData.effect)}
            type={smearData.smearMode}
            col={6}
            smearedColor={{ h: smearData.hue, s: smearData.saturation, v: smearData.value }}
            lightColorMaps={smearLightColorMaps}
            // onLightChange={onLightChange}
            // onLightEnd={onLightEnd}
            onLightTap={onLightEnd}
          />
        </View>
      </View>


      <Popup
        show={isShowSave}
        customStyle={{ background: "#FFF" }}
        round
        closeOnClickOverlay={false}
        onClose={() => {
          setIsShowSave(false);
        }}
      >
        <View className={styles.save_box}>
          <View className={styles.save_box_top}>{String.getLang("edit_name")}</View>
          <View className={styles.save_box_center}>
            <Input className={styles.input} placeholder={String.getLang("enter_name")} maxLength={8} onInput={(e) => {
              setName(e.detail.value)
            }} />
          </View>
          <View className={styles.save_box_bottom}>
            <View className={styles.btn} onClick={() => {
              setIsShowSave(false);
            }}>{String.getLang("cancel")}</View>
            <View className={clsx(styles.btn, styles.comfirm)}
              onClick={() => {
                handleComfirmSave()
              }}>{String.getLang("confirm")}</View>
          </View>
        </View>
      </Popup>

    </View>
  )
}

export default memo(SmearLights);
